package framework.dialogs
{
	import assets.ButtonEx;
	
	import flash.display.CapsStyle;
	import flash.display.JointStyle;
	import flash.display.LineScaleMode;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.filters.DropShadowFilter;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFieldType;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import framework.constants.COMMONSTRINGS;
	
	import mx.controls.Button;
	import mx.core.Application;
	import mx.core.UIComponent;
	import mx.effects.Blur;
	import mx.effects.Parallel;
	import mx.effects.Zoom;
	import mx.effects.easing.Bounce;
	import mx.events.CloseEvent;
	import mx.utils.object_proxy;
	
	

	public class MessageBox extends UIComponent
	{
		//--------------------------------------------------------------------------
	    //
	    //  Class constants
	    //
	    //--------------------------------------------------------------------------
	
	    /**
	     *  Value that enables a Yes button on the Alert control when passed
	     *  as the <code>flags</code> parameter of the <code>show()</code> method.
	     *  You can use the | operator to combine this bitflag
	     *  with the <code>OK</code>, <code>CANCEL</code>,
	     *  <code>NO</code>, and <code>NONMODAL</code> flags.
	     */
	    public static const YES:uint = 0x0001;
	    
	    /**
	     *  Value that enables a No button on the Alert control when passed
	     *  as the <code>flags</code> parameter of the <code>show()</code> method.
	     *  You can use the | operator to combine this bitflag
	     *  with the <code>OK</code>, <code>CANCEL</code>,
	     *  <code>YES</code>, and <code>NONMODAL</code> flags.
	     */
	    public static const NO:uint = 0x0002;
	    
	    /**
	     *  Value that enables an OK button on the Alert control when passed
	     *  as the <code>flags</code> parameter of the <code>show()</code> method.
	     *  You can use the | operator to combine this bitflag
	     *  with the <code>CANCEL</code>, <code>YES</code>,
	     *  <code>NO</code>, and <code>NONMODAL</code> flags.
	     */
	    public static const OK:uint = 0x0004;
	    
	    /**
	     *  Value that enables a Cancel button on the Alert control when passed
	     *  as the <code>flags</code> parameter of the <code>show()</code> method.
	     *  You can use the | operator to combine this bitflag
	     *  with the <code>OK</code>, <code>YES</code>,
	     *  <code>NO</code>, and <code>NONMODAL</code> flags.
	     */
	    public static const CANCEL:uint= 0x0008;
	
	    /**
	     *  Value that makes an Alert nonmodal when passed as the
	     *  <code>flags</code> parameter of the <code>show()</code> method.
	     *  You can use the | operator to combine this bitflag
	     *  with the <code>OK</code>, <code>CANCEL</code>,
	     *  <code>YES</code>, and <code>NO</code> flags.
	     */
	  //  public static const NONMODAL:uint = 0x8000;
	    
	    //----------------------------------
	    //  buttonFlags
	    //----------------------------------
	
	    /**
	     *  A bitmask that contains <code>Alert.OK</code>, <code>Alert.CANCEL</code>, 
	     *  <code>Alert.YES</code>, and/or <code>Alert.NO</code> indicating
		 *  the buttons available in the Alert control.
	     *
	     *  @default Alert.OK
	     */
	    public var buttonFlags:uint = OK;
	    
	    //----------------------------------
	    //  defaultButtonFlag
	    //----------------------------------
	
	    [Inspectable(category="General")]
	
	    /**
	     *  A bitflag that contains either <code>Alert.OK</code>, 
	     *  <code>Alert.CANCEL</code>, <code>Alert.YES</code>, 
	     *  or <code>Alert.NO</code> to specify the default button.
	     *
	     *  @default Alert.OK
	     */
	    public var defaultButtonFlag:uint = OK;
	    
	    //----------------------------------
	    //  iconClass
	    //----------------------------------
	
	    [Inspectable(category="Other")]
	
	    /**
	     *  The class of the icon to display.
	     *  You typically embed an asset, such as a JPEG or GIF file,
	     *  and then use the variable associated with the embedded asset 
	     *  to specify the value of this property.
	     *
	     *  @default null
	     */
	    public var iconClass:Class;
	    
		private var dialogBoxWidth:int =250;
		private var dialogBoxHeight:int = 170;
		
		private var okBtn:ButtonEx;
		private var cancelBtn:ButtonEx
		
		private var header:TextField;
		private var message:TextField;
		
		private var customHandler:Function = null;
		
		private static var internalObject:MessageBox = null;
		
		private var effects:Parallel;
		
		private var dialogBox:Sprite;
		
		public function MessageBox()
		{
			super();
			this.percentWidth = 100;
			this.percentHeight = 100;
		}
		
		protected override function createChildren():void
		{
			dialogBox = new Sprite();
			
			this.addChild(dialogBox);
		
			effects = new Parallel();
			var blurEffect:Blur =  new Blur();
			blurEffect.blurXFrom = 125;
			blurEffect.blurYFrom = 75;
			blurEffect.blurXTo = 0;
			blurEffect.blurYTo = 0;
			effects.addChild(blurEffect);
			
			var zoomEffect:Zoom = new Zoom();
			 zoomEffect.zoomWidthFrom = 0.01;
			 zoomEffect.zoomHeightFrom = 0.01;
			 zoomEffect.zoomWidthTo = 1.0;
			 zoomEffect.zoomHeightTo = 1.0;
			 zoomEffect.easingFunction = Bounce.easeInOut;
			 effects.addChild(zoomEffect);
			
			okBtn = new ButtonEx();
			okBtn.setActualSize(60,25);
			okBtn.addEventListener(MouseEvent.CLICK,clickHandler);
			this.addChild(okBtn);
			
			
			cancelBtn = new ButtonEx();
			cancelBtn.setActualSize(80,25);
			cancelBtn.addEventListener(MouseEvent.CLICK,clickHandler);
			this.addChild(cancelBtn);
			
			var headerformat:TextFormat = new TextFormat();
			headerformat.font = getStyle("titleFontFamily");
			headerformat.color = getStyle("titleFontColor");
			headerformat.size = getStyle("titleFontSize");			
			headerformat.bold = true;
			//textBox.defaultTextFormat = format;
			//textBox.autoSize = TextFieldAutoSize.LEFT;
			
			//textBox.setTextFormat(format);				
			
			header = new TextField();
			header.defaultTextFormat = headerformat;
			//header.width = 150;
			header.autoSize = TextFieldAutoSize.LEFT;
			header.border=false;
			header.type= TextFieldType.DYNAMIC;
			header.selectable=false;
			header.mouseEnabled=false;
			this.addChild(header);
			
			var messageformat:TextFormat = new TextFormat();
			messageformat.font = getStyle("bodyFontFamily");
			messageformat.color = getStyle("bodyFontColor");
			messageformat.size = getStyle("bodyFontSize");
			messageformat.bold = false;	
			messageformat.align = TextFormatAlign.CENTER;
			
			message = new TextField();
			message.defaultTextFormat = messageformat;
			message.autoSize = TextFieldAutoSize.LEFT;
			message.border=false;
			message.type= TextFieldType.DYNAMIC;
			message.selectable=false;
			message.mouseEnabled=false;
			message.wordWrap = true;
			
			this.addChild(message);
						
			this.systemManager.stage.focus = okBtn;
			
			var borderSize:int = getStyle("borderThickness");
			dialogBox.graphics.lineStyle(borderSize,getStyle("borderColor"),1,true, LineScaleMode.NORMAL,
                               CapsStyle.ROUND, JointStyle.ROUND, 3);			
			
			dialogBox.graphics.beginFill(getStyle("dialogColor"),1);
			dialogBox.graphics.drawRoundRect(0 ,0 , dialogBoxWidth, dialogBoxHeight,25,25);
			
			
			dialogBox.graphics.lineStyle(1,getStyle("strokeColor"),1,true);
			dialogBox.graphics.moveTo(5, 5 + 25 + 5);
			dialogBox.graphics.lineTo(dialogBoxWidth-5,5 + 25 + 5);
			var shadow:DropShadowFilter = new DropShadowFilter();
			shadow.color = getStyle("shadowColor");
			shadow.distance = 7;
			shadow.angle = 54;
			shadow.blurX = 11;
			shadow.blurY = 12;
			dialogBox.filters = [shadow];
			
			message.width = dialogBox.width - 40;
		    message.height = dialogBox.height - header.height - okBtn.height - 50;
			
			setText();
		}
		
		public static function setText():void
		{
			if(internalObject != null)
			{
			//	internalObject.message.text = COMMONSTRINGS.ERROR_DELETE_CONFIRMATION;
			//	internalObject.header.text = COMMONSTRINGS.MESSAGEBOX_TITLE_ALERT;
				internalObject.okBtn.label = COMMONSTRINGS.BUTTON_OK;
				internalObject.cancelBtn.label = COMMONSTRINGS.BUTTON_CANCEL;
			}
		}
		
				
		protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{	
			dialogBox.x = (unscaledWidth-dialogBox.width)/2;
			dialogBox.y = (unscaledHeight-dialogBox.height)/2;
			
			this.graphics.clear();
			this.graphics.beginFill(getStyle("backgroundColor"),getStyle("backgroundAlpha"));
			this.graphics.drawRect(0, 0, unscaledWidth, unscaledHeight);
			
		    okBtn.y = dialogBox.y + dialogBoxHeight - okBtn.height -10;
		    cancelBtn.y = okBtn.y;
		    
		    if(cancelBtn.visible)
		    {
		    	okBtn.x = dialogBox.x +(dialogBoxWidth-okBtn.width-cancelBtn.width -20)/2;
		    }
		    else
		    {
		    	okBtn.x = dialogBox.x +(dialogBoxWidth-okBtn.width)/2;		    	
		    }
		    
		    
		    cancelBtn.x = okBtn.x +okBtn.width+20;
		    	
		    header.x = dialogBox.x + 5;
		    header.y = dialogBox.y + 5;
		    
		 
		    message.x = header.x;
		    message.y = header.y + header.height + 15;
		    
		}
		
		public static function Show(message:String,headertext:String="",
                                closeHandler:Function = null, 
                                flags:uint = 0x4 /* MessageBox.OK */, 
                                iconClass:Class = null, 
                                defaultButtonFlag:uint = 0x4 /* MessageBox.OK */):void
		{
			// var modal:Boolean = (flags & MessageBox.NONMODAL) ? false : true;
			
			if(internalObject == null)
			{
				internalObject = new MessageBox();	
				Application.application.addChild(internalObject);			
			}
			else
			{
				Application.application.removeChild(internalObject);	
				Application.application.addChild(internalObject);	
			}
			
			internalObject.Init(message,headertext,flags,closeHandler,iconClass,defaultButtonFlag);
		}
		
		private function Init(messagetext:String,headertext:String="",
                                flags:uint = 0x4 /* MessageBox.OK */, 
                                closeHandler:Function = null, 
                                iconClass:Class = null, 
                                defaultButtonFlag:uint = 0x4 /* MessageBox.OK */):void
		{
			
			if (flags & MessageBox.OK||
            flags & MessageBox.CANCEL||
            flags & MessageBox.YES||
            flags & MessageBox.NO)
	        {
	            this.buttonFlags = flags;
	        }
	        
	        if (defaultButtonFlag == MessageBox.OK ||
	            defaultButtonFlag == MessageBox.CANCEL||
	            defaultButtonFlag & MessageBox.YES||
	            defaultButtonFlag & MessageBox.NO )
	        {
	            this.defaultButtonFlag = defaultButtonFlag;
	        }

			if(headertext == "")
			{
				header.text = COMMONSTRINGS.MESSAGEBOX_TITLE_ALERT;
			}			
			else
			{				
				header.text = headertext;			
			}
			
			if ((buttonFlags & MessageBox.OK) || (buttonFlags & MessageBox.YES))
			{
				okBtn.visible = true;
				if(buttonFlags & MessageBox.YES)
				{
					okBtn.label = COMMONSTRINGS.BUTTON_YES;
				}
				else
				{
					okBtn.label = COMMONSTRINGS.BUTTON_OK;
				}
			}
			else
			{
				okBtn.visible = false;				
			}
			
			if (buttonFlags & MessageBox.CANCEL|| (buttonFlags & MessageBox.NO))
			{
				cancelBtn.visible = true;
				if(buttonFlags & MessageBox.NO)
				{
					cancelBtn.label = COMMONSTRINGS.BUTTON_NO;
				}
				else
				{
					cancelBtn.label = COMMONSTRINGS.BUTTON_CANCEL;
				}
			}
			else
			{
				cancelBtn.visible = false;				
			}
			
			message.text = messagetext;
			
			customHandler = closeHandler;
			
			this.visible = true;
			//effects.play([this]);
		}
		
		private function clickHandler(evt:MouseEvent):void
		{
			this.visible = false;
			
			if(customHandler != null)
			{
				if(Button(evt.currentTarget).label == COMMONSTRINGS.BUTTON_OK)
				{
					customHandler(new CloseEvent(CloseEvent.CLOSE,false,false,MessageBox.OK));
				}
				else if(Button(evt.currentTarget).label == COMMONSTRINGS.BUTTON_CANCEL)
				{
					customHandler(new CloseEvent(CloseEvent.CLOSE,false,false,MessageBox.CANCEL));
				}
				else if(Button(evt.currentTarget).label == COMMONSTRINGS.BUTTON_YES)
				{
					customHandler(new CloseEvent(CloseEvent.CLOSE,false,false,MessageBox.YES));
				}
				else if(Button(evt.currentTarget).label == COMMONSTRINGS.BUTTON_NO)
				{
					customHandler(new CloseEvent(CloseEvent.CLOSE,false,false,MessageBox.NO));
				}
			}
		}
		
		
	}
}