package modules.batchcataloger.UI
{
	import assets.ButtonEx;
	
	import flash.desktop.ClipboardFormats;
	import flash.desktop.NativeDragManager;
	import flash.events.FileListEvent;
	import flash.events.MouseEvent;
	import flash.events.NativeDragEvent;
	import flash.filesystem.File;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFieldType;
	import flash.text.TextFormat;
	
	import modules.batchcataloger.events.CatalogJobDetailEvent;
	import modules.batchcataloger.framework.constants.STRINGS;
	
	import mx.containers.Panel;
	import mx.core.UIComponent;
	
	public class MultiFilesDropArea extends UIComponent
	{
		private var holder:Panel;
		
		private var addFilesSection:UIComponent;
		private var addFilesErrorSection:UIComponent;
		
		private var uploadInfo:TextField;
		private var selectInfo:TextField;
		
		private var errorLabel:TextField;
		private var errorDetailsLabel:TextField;
		private var errorFilesLabel:TextField;
		
		private var selectFilesBtn:ButtonEx;
		private var cancelBtn:ButtonEx;
		
		private var errorCancelbtn:ButtonEx;
		
		private var isPreview:Boolean;
		
		private var file:File;
		
		public function MultiFilesDropArea(dropPreview:Boolean=false)
		{
			super();
			
			isPreview = dropPreview;
			
			addFilesSection = new UIComponent();
			addFilesErrorSection = new UIComponent();
			
			file = new File();
			file.addEventListener(FileListEvent.SELECT_MULTIPLE, onSelectHandler);
			
			this.addEventListener(NativeDragEvent.NATIVE_DRAG_ENTER,onDragIn);
          	this.addEventListener(NativeDragEvent.NATIVE_DRAG_DROP,onDrop);
         	this.addEventListener(NativeDragEvent.NATIVE_DRAG_EXIT,onDragExit);
         	this.addEventListener(CatalogJobDetailEvent.UNMATCHED_FILES_ERROR, showErrorHandler);
		}
		
		protected override function createChildren():void
		{
			holder = new Panel();
			
			holder.styleName = "holderStyle";
			if (isPreview)
				holder.title = STRINGS.LABEL_ADD_PREVIEWS;
			else
				holder.title = STRINGS.LABEL_ADD_THUMBNAILS;
			holder.layout = "vertical";
			
			this.addChild(holder);
						
			this.addChild(addFilesSection);
			
			var labelformat:TextFormat = new TextFormat();
			labelformat.font = getStyle("labelFontFamily");
			labelformat.color = getStyle("labelFontColor");
			labelformat.size = getStyle("labelFontSize");
			labelformat.bold = true;	
			
			uploadInfo = new TextField();
			uploadInfo.defaultTextFormat = labelformat;
			uploadInfo.autoSize = TextFieldAutoSize.LEFT;
			uploadInfo.border=false;
			uploadInfo.type= TextFieldType.DYNAMIC;
			uploadInfo.selectable=false;
			uploadInfo.mouseEnabled=false;
			if (isPreview)
				uploadInfo.text = STRINGS.LABEL_UPLOAD_PREVIEWS_INFO; 
			else
				uploadInfo.text = STRINGS.LABEL_UPLOAD_THUMBNAILS_INFO;
			
			addFilesSection.addChild(uploadInfo);
			
			labelformat.bold = false;
			labelformat.italic = true;
			
			selectInfo = new TextField();
			selectInfo.defaultTextFormat = labelformat;
			selectInfo.autoSize = TextFieldAutoSize.LEFT;
			selectInfo.border=false;
			selectInfo.type= TextFieldType.DYNAMIC;
			selectInfo.selectable=false;
			selectInfo.mouseEnabled=false;
			if (isPreview)
				selectInfo.text = STRINGS.LABEL_SELECT_PREVIEWS_INFO; 
			else
				selectInfo.text = STRINGS.LABEL_SELECT_THUMBNAILS_INFO;
			
			addFilesSection.addChild(selectInfo);
			
			selectFilesBtn = new ButtonEx();
			if (isPreview)
				selectFilesBtn.label = STRINGS.BUTTON_SELECT_PREVIEWS;
			else
				selectFilesBtn.label = STRINGS.BUTTON_SELECT_THUMBNAILS;
			selectFilesBtn.setActualSize(140, 25);
			
			addFilesSection.addChild(selectFilesBtn);
			
			selectFilesBtn.addEventListener(MouseEvent.CLICK, selectFilesHandler);
			
			cancelBtn = new ButtonEx();
			cancelBtn.label = STRINGS.BUTTON_ADD_FILES_CANCEL;
			cancelBtn.setActualSize(80, 25);
			
			addFilesSection.addChild(cancelBtn);
			
			cancelBtn.addEventListener(MouseEvent.CLICK, cancelClickHandler);
			
			addFilesErrorSection.visible = false;
			this.addChild(addFilesErrorSection);
			
			var errorLabelformat:TextFormat = new TextFormat();
			errorLabelformat.font = getStyle("errorFontFamily");
			errorLabelformat.color = getStyle("errorFontColor");
			errorLabelformat.size = getStyle("errorFontSize");
			errorLabelformat.bold = true;
			
			errorLabel = new TextField();
			errorLabel.defaultTextFormat = errorLabelformat;
			errorLabel.autoSize = TextFieldAutoSize.LEFT;
			errorLabel.border=false;
			errorLabel.type= TextFieldType.DYNAMIC;
			errorLabel.selectable=false;
			errorLabel.mouseEnabled=false;
			errorLabel.text = STRINGS.LABEL_ADD_FILES_ERROR;
			
			addFilesErrorSection.addChild(errorLabel);
			
			errorDetailsLabel = new TextField();
			errorDetailsLabel.defaultTextFormat = errorLabelformat;
			errorDetailsLabel.autoSize = TextFieldAutoSize.LEFT;
			errorDetailsLabel.border=false;
			errorDetailsLabel.type= TextFieldType.DYNAMIC;
			errorDetailsLabel.selectable=false;
			errorDetailsLabel.mouseEnabled=false;
			if (isPreview)
				errorDetailsLabel.text = STRINGS.LABEL_ADD_PREVIEWS_ERROR_DETAIL;
			else
				errorDetailsLabel.text = STRINGS.LABEL_ADD_THUMBNAILS_ERROR_DETAIL;
			
			addFilesErrorSection.addChild(errorDetailsLabel);
			
			errorLabelformat.bold = false;
			errorLabelformat.bullet = true;
			
			errorFilesLabel = new TextField();
			errorFilesLabel.visible = false;
			errorFilesLabel.defaultTextFormat = errorLabelformat;
			errorFilesLabel.autoSize = TextFieldAutoSize.LEFT;
			errorFilesLabel.border=false;
			errorFilesLabel.type= TextFieldType.DYNAMIC;
			errorFilesLabel.selectable=false;
			errorFilesLabel.mouseEnabled=false;
			
			addFilesErrorSection.addChild(errorFilesLabel);
			
			errorCancelbtn = new ButtonEx();
			errorCancelbtn.label = STRINGS.BUTTON_ADD_FILES_ERROR_CANCEL;
			errorCancelbtn.setActualSize(60, 25);
					
			addFilesErrorSection.addChild(errorCancelbtn);
			
			errorCancelbtn.addEventListener(MouseEvent.CLICK, errorCancelClickHandler);
		}
		
		public function setText():void
		{
			if(holder == null) return;
			
			if (isPreview)
			{
				holder.title = STRINGS.LABEL_ADD_PREVIEWS;
				uploadInfo.text = STRINGS.LABEL_UPLOAD_PREVIEWS_INFO; 
				selectInfo.text = STRINGS.LABEL_SELECT_PREVIEWS_INFO; 
				selectFilesBtn.label = STRINGS.BUTTON_SELECT_PREVIEWS;
				errorDetailsLabel.text = STRINGS.LABEL_ADD_PREVIEWS_ERROR_DETAIL;
			}
			else
			{
				holder.title = STRINGS.LABEL_ADD_THUMBNAILS;
				uploadInfo.text = STRINGS.LABEL_UPLOAD_THUMBNAILS_INFO;
				selectInfo.text = STRINGS.LABEL_SELECT_THUMBNAILS_INFO;
				selectFilesBtn.label = STRINGS.BUTTON_SELECT_THUMBNAILS;
				errorDetailsLabel.text = STRINGS.LABEL_ADD_THUMBNAILS_ERROR_DETAIL;
			}
			
			cancelBtn.label = STRINGS.BUTTON_ADD_FILES_CANCEL;
			
			errorLabel.text = STRINGS.LABEL_ADD_FILES_ERROR;
			errorCancelbtn.label = STRINGS.BUTTON_ADD_FILES_ERROR_CANCEL;
		}
		
		protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			this.graphics.clear();
			this.graphics.beginFill(getStyle("backgroundColor"),1);
			this.graphics.drawRect(0, 0, unscaledWidth, unscaledHeight);
			
			/*addFilesErrorSection.graphics.clear();
			addFilesErrorSection.graphics.beginFill(getStyle("errorbackgroundColor"),1);
			addFilesErrorSection.graphics.drawRect(0, 0, unscaledWidth, unscaledHeight);*/
			
			holder.width = unscaledWidth;
			holder.height = unscaledHeight;
			
			uploadInfo.x = holder.x + 10;
			uploadInfo.y = holder.y + 40;
			
			errorLabel.x = uploadInfo.x;
			errorLabel.y = uploadInfo.y;
			
			selectInfo.x = uploadInfo.x;
			selectInfo.y = uploadInfo.y + 30;
			
			errorDetailsLabel.x = selectInfo.x;
			errorDetailsLabel.y = selectInfo.y;
			
			selectFilesBtn.x = uploadInfo.x;
			selectFilesBtn.y = selectInfo.y + 30;
			
			cancelBtn.x = selectFilesBtn.x + selectFilesBtn.width + 10;
			cancelBtn.y = selectFilesBtn.y;
			
			errorFilesLabel.x = selectInfo.x;
			errorFilesLabel.y = selectInfo.y + 40;
			
			errorCancelbtn.x = errorFilesLabel.x;
			errorCancelbtn.y = errorFilesLabel.y + errorFilesLabel.textHeight + 20;
		}
		
		public function Init():void
		{
			addFilesErrorSection.visible = false;
			addFilesSection.visible = true;
		}
		
		private function onDragIn(event:NativeDragEvent):void
		{
			var files:Array = event.clipboard.getData(ClipboardFormats.FILE_LIST_FORMAT) as Array;
				    
			if(files != null && files.length > 0)
			{
				NativeDragManager.acceptDragDrop(this);
			}
		}
		
		private function onDrop(event:NativeDragEvent):void
        {
        	var files:Array = event.clipboard.getData(ClipboardFormats.FILE_LIST_FORMAT) as Array;
        	if(files != null && files.length > 0)
        		dispatchEvent(new CatalogJobDetailEvent(CatalogJobDetailEvent.PREVIEWS_THUMBNAILS_DROPPED, files));
        }
        
        private function onDragExit(event:NativeDragEvent):void
		{
			
		}
		
		private function selectFilesHandler(evt:MouseEvent):void
		{	
			file.browseForOpenMultiple("Open");
		}
		
		private function onSelectHandler(evt:FileListEvent):void
		{
			var files:Array = evt.files;
				 
			if (files != null && files.length > 0)
				dispatchEvent(new CatalogJobDetailEvent(CatalogJobDetailEvent.PREVIEWS_THUMBNAILS_DROPPED, files));
		}
		
		private function showErrorHandler(evt:CatalogJobDetailEvent):void
		{
			var unMatchedFiles:Array = evt.Data as Array;
			var unMatchedFilesList:String = "";
			for (var i:int = 0; i < unMatchedFiles.length; i++)
			{
				unMatchedFilesList += unMatchedFiles[i];
   				if (i != unMatchedFiles.length-1)
   					unMatchedFilesList += "\n";
   			}
			addFilesSection.visible = false;
			addFilesErrorSection.visible = true;
			errorFilesLabel.text = unMatchedFilesList;
			errorFilesLabel.visible = true;
			invalidateDisplayList();
		}
		
		private function errorCancelClickHandler(evt:MouseEvent):void
		{
			dispatchEvent(new CatalogJobDetailEvent(CatalogJobDetailEvent.UNMATCHED_FILES_ERROR_CANCEL));
		}
		
		private function cancelClickHandler(evt:MouseEvent):void
		{
			dispatchEvent(new CatalogJobDetailEvent(CatalogJobDetailEvent.PREVIEWS_THUMBNAILS_DROP_CANCEL));
		}
	}
}