package modules.batchcataloger.UI
{
	import assets.ButtonEx;
	
	import flash.display.CapsStyle;
	import flash.display.JointStyle;
	import flash.display.LineScaleMode;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.filters.DropShadowFilter;
	
	import framework.dialogs.MessageBox;
	import framework.utils.Helper;
	
	import infrastructure.statemanagement.StateMgr;
	
	import modules.batchcataloger.assets.TreeItemRendererEx;
	import modules.batchcataloger.events.TargetLocationEvent;
	import modules.batchcataloger.framework.constants.COMMON;
	import modules.batchcataloger.framework.constants.STRINGS;
	
	import mx.collections.XMLListCollection;
	import mx.controls.Tree;
	import mx.core.ClassFactory;
	import mx.core.UIComponent;
	import mx.events.FlexEvent;
	import mx.events.ListEvent;

	public class TargetSelector extends UIComponent
	{
		private var dialogBoxWidth:int = 350;
		private var dialogBoxHeight:int = 370;
		private var dialogBox:Sprite;
		 
		private var cancelBtn:ButtonEx;
		private var createSectionBtn:ButtonEx;
		private var selectBtn:ButtonEx;
		
		private var createSectionPopup:CreateSection;
		
		private var isSectionCreated:Boolean;
		
		private var tree:Tree;
		[Bindable]
        public var sections:XML;
              
		[Bindable]
        private var sectionHierarchy:XMLListCollection ;
            
		private var cacheFolder:File;
		private var file:File;
		
		public function TargetSelector()
		{
			super();			
			createSectionPopup = new CreateSection();
			createSectionPopup.Hide();
			
			this.percentHeight = 100;
			this.percentWidth = 100;	
			cacheFolder = Helper.CreateCacheDirectory(COMMON.CACHE_FOLDERNAME +"/" +StateMgr.GetInstance().UserId );
			
			file = new File(cacheFolder.nativePath + "/" + COMMON.FILE_SECTION_HIERARCHY);
			
			try
			{
				var fs:FileStream = new FileStream();
				fs.open(file, FileMode.READ);
				sections = XML(fs.readUTFBytes(fs.bytesAvailable));
				fs.close();
			}
			catch(ex:Error)
			{
				MessageBox.Show(STRINGS.ERROR_LOADING_SECTIONS);
				StateMgr.GetInstance().logger.Write("TargetSelector","Constructor",ex.toString());
				
			}
			
			sectionHierarchy = new XMLListCollection(sections.TreeNode);
			this.styleName = "targetSelectorStyle";
         }
		
		protected override function createChildren():void
		{
			super.createChildren();
			
			dialogBox = new Sprite();
			var borderSize:int = getStyle("borderThickness");
			dialogBox.graphics.lineStyle(borderSize,getStyle("borderColor"),1,true, LineScaleMode.NORMAL,
                               CapsStyle.ROUND, JointStyle.ROUND, 3);			
			
			dialogBox.graphics.beginFill(getStyle("dialogColor"),1);
			dialogBox.graphics.drawRoundRect(0 ,0 , dialogBoxWidth, dialogBoxHeight,25,25);					
			
			var shadow:DropShadowFilter = new DropShadowFilter();
			shadow.color = getStyle("shadowColor");
			shadow.distance = 7;
			shadow.angle = 54;
			shadow.blurX = 11;
			shadow.blurY = 12;
			dialogBox.filters = [shadow];
			
			this.addChild(dialogBox);
			
			tree = new Tree();
			tree.itemRenderer = new ClassFactory(TreeItemRendererEx);
			tree.height =  dialogBoxHeight - 50;
			tree.width = dialogBoxWidth - 20;
			tree.labelFunction = treeLabel;
			tree.styleName = "targetSelectorTreeStyle"; 
			//tree.iconFunction = getTreeIcon;
			//tree.labelField = "@Text";
			tree.dataProvider = sectionHierarchy;
			
			this.addChild(tree);
			tree.setStyle("defaultLeafIcon",tree.getStyle("folderClosedIcon"));
			
			//tree.addEventListener(ListEvent.CHANGE,selectedTargetHandler);	
			tree.addEventListener(FlexEvent.CREATION_COMPLETE,initTree);
			//tree.addEventListener(TreeEvent.ITEM_OPEN, treeItemOpenHandler);
			//tree.addEventListener(TreeEvent.ITEM_CLOSE, treeItemCloseHandler);
			
			selectBtn = new ButtonEx();
			selectBtn.label = STRINGS.BUTTON_TARGETLOCATION_SELECT;
			selectBtn.setActualSize(80,25);
			this.addChild(selectBtn);
			
			cancelBtn = new ButtonEx();
			cancelBtn.label = STRINGS.BUTTON_TARGETLOCATION_CANCEL;
			cancelBtn.setActualSize(80,25);
			this.addChild(cancelBtn);
			
			createSectionBtn = new ButtonEx();
			createSectionBtn.label = STRINGS.BUTTON_TARGETLOCATION_CREATESECTION;
			createSectionBtn.setActualSize(120,25);
			this.addChild(createSectionBtn);
			
			this.addChild(createSectionPopup);
			
			createSectionPopup.addEventListener(TargetLocationEvent.CREATE_SECTION_DONE, createSectionDoneHandler);
			
//			createSectionBtn.addEventListener(MouseEvent.CLICK, CreateSectionHandler);
			createSectionBtn.addEventListener(MouseEvent.CLICK, createSectionClickHandler);
			selectBtn.addEventListener(MouseEvent.CLICK,selectClickHandler);
			cancelBtn.addEventListener(MouseEvent.CLICK,cancelClickHandler);
		}
		
		public function getTreeIcon(item:Object):Class
		{	
			return tree.getStyle("folderClosedIcon");
		} 
		
		protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{	
			createSectionPopup.width = unscaledWidth;
			createSectionPopup.height = unscaledHeight;
			
			dialogBox.x = (unscaledWidth-dialogBox.width)/2;
			dialogBox.y = (unscaledHeight-dialogBox.height)/2;
			
			this.graphics.clear();
			this.graphics.beginFill(getStyle("backgroundColor"),getStyle("backgroundAlpha"));
			this.graphics.drawRect(0, 0, unscaledWidth, unscaledHeight);
						
			
			tree.x = (unscaledWidth - tree.width)/2;
			tree.y = dialogBox.y + 10;//(unscaledHeight - tree.height)/2;
			
			createSectionBtn.x = dialogBox.x +(dialogBoxWidth-createSectionBtn.width) -10;
			createSectionBtn.y = dialogBox.y + tree.height + 15;
			
			cancelBtn.x = createSectionBtn.x - cancelBtn.width - 40;
			cancelBtn.y = createSectionBtn.y;
			
			selectBtn.x = cancelBtn.x - selectBtn.width - 10;
			selectBtn.y = createSectionBtn.y;
			
			//createSectionForm.x = (unscaledWidth - createSectionForm.width)/2;
			//createSectionForm.y = createSectionBtn.y + createSectionBtn.height + 5; 
		}
		
		private function initTree(evt:FlexEvent):void
		{			
			  traverseTree(sections.children());
		}
		
		private function traverseTree(nodeList:XMLList):void
		{
			for ( var i:Number = 0 ; i < nodeList.length() ; i++ )
			  {
			  	   var str:String = nodeList[i].@NodeData;
			    	StateMgr.GetInstance().HashedSectionHierarchy[str] = nodeList[i];
			        traverseTree(nodeList[i].children());
			  }    
		}
		
		private function treeLabel(item:Object):String
        {

            var node:XML = XML(item);
            if( node.localName() == "TreeNode" )

                return node.@Text;
            else
                return "";
        }
		
		private function selectClickHandler(evt:MouseEvent):void
		{
			if (tree.selectedItem != null)
			{
				this.dispatchEvent(new TargetLocationEvent(TargetLocationEvent.TARGET_CHANGE,tree.selectedItem));
				this.visible = false;
			}
			else
				MessageBox.Show(STRINGS.ERROR_SELECT_SECTION);
			
		}
		
		private function selectedTargetHandler(evt:ListEvent):void
		{	
			//if (createSectionForm.visible == false)
			{
				this.dispatchEvent(new TargetLocationEvent(TargetLocationEvent.TARGET_CHANGE,evt.currentTarget.selectedItem));
				this.visible = false;
			}
			/*else
			{
				pathInput.text = XML(evt.currentTarget.selectedItem).@Path.toString();
			}*/
		}
		
		private function cancelClickHandler(evt:MouseEvent):void
		{
			this.visible = false;			
		}
		
		public function Show(id:String=""):void
		{	
			this.visible = true;
			//initCreateSectionForm();
			//hideCreateSectionForm();
			collapseTree();
			if(id != "")
			{			
				var node:XML = StateMgr.GetInstance().HashedSectionHierarchy[id];
				if(node != null)
				{	
					expandParentsOf(StateMgr.GetInstance().HashedSectionHierarchy[id]);				
					tree.selectedItem = StateMgr.GetInstance().HashedSectionHierarchy[id];
					//pathInput.text = StateMgr.GetInstance().HashedSectionHierarchy[id].@Path;
				}
			}
		}
		
		private function collapseTree():void
		{  	
	        for (var i:int = 0; i < sectionHierarchy.length; i ++)
	        {
	                tree.expandChildrenOf(sectionHierarchy[i], false)
	        }
        } 
        
        private function expandTree():void
		{
                for (var i:int = 0; i < sectionHierarchy.length; i ++)
                {
                        tree.expandChildrenOf(sectionHierarchy[i], true)
                }
        } 
        
        private function expandParentsOf(node:XML):void
		{
			
			while(node.parent().localName() != COMMON.SH_XML_ROOT_NODE )
			{
				tree.expandItem(node.parent(),true);
				node = node.parent();				
			}
		}
		
		/*********** The following functions are used for section creation**********/
		
		private function createSectionClickHandler(evt:MouseEvent):void
		{
			if (tree.selectedItem != null)
			{	
				createSectionPopup.dispatchEvent(new TargetLocationEvent(TargetLocationEvent.CREATE_SECTION_OPEN, tree.selectedItem.@Path));
				createSectionPopup.Init();
				createSectionPopup.Show();	
			}
			else
			{
				MessageBox.Show(STRINGS.ERROR_SELECT_SECTION);
			}
		}
		
		private function createSectionDoneHandler(evt:TargetLocationEvent):void
		{
			var sectionName:String = evt.Data as String;
			AddSection(sectionName);
		}
		
		private function AddSection(sectionName:String):void
		{
           	var newSection:XML;
           	var sectionNameArr:Array = sectionName.split('\\');
           	var currPath:String = tree.selectedItem.@Path;
       		var treeNodeList:XMLList = sections.TreeNode;
			while (treeNodeList != null && treeNodeList.length() > 0)
			{	
				var selectedNode:XML = treeNodeList.(String(@Path) == currPath)[0];
				if (selectedNode != null)
				{	
					var existingChilds:XMLList;
					for (var i:int = 0; i < sectionNameArr.length; i++)
					{
						existingChilds = selectedNode.TreeNode.(String(@Path)==currPath+'\\'+sectionNameArr[i]);
						sectionName = sectionNameArr[i];
						//following condition is to check whether section already exists or not
						if (existingChilds == null || existingChilds.length() == 0)
						{
							var currDateTimeStamp:String = Helper.GetCurrentDateTimeStamp() + i.toString();
							var encName:String = Helper.EncodeSpecialChars(sectionName);
							newSection = XML("<TreeNode Text='"+encName+"' Path='"+Helper.EncodeSpecialChars(currPath)+'\\'+encName+"' NodeData='"+currDateTimeStamp+"' Status='new' />");
							selectedNode.appendChild(newSection);
							StateMgr.GetInstance().HashedSectionHierarchy[currDateTimeStamp] = newSection;
							tree.expandItem(selectedNode, true);
							var selNodeChilds:XMLList = selectedNode.TreeNode;
							selectedNode = selNodeChilds[selNodeChilds.length() - 1];
							currPath += '\\' + sectionName;
							isSectionCreated = true;
							//currPath += '\\' + encName;
						}
						else
						{	
							tree.expandItem(selectedNode, true);
							selectedNode = existingChilds[0];
							currPath = selectedNode.@Path;
						}
					}
					if (isSectionCreated)
						this.dispatchEvent(new TargetLocationEvent(TargetLocationEvent.SECTION_CREATED));
					break;
				}
				treeNodeList = treeNodeList.TreeNode;
			}
		}	
		
		public function setText():void
		{
			selectBtn.label 		= STRINGS.BUTTON_TARGETLOCATION_SELECT;			
			cancelBtn.label 		= STRINGS.BUTTON_TARGETLOCATION_CANCEL;			
			createSectionBtn.label 	= STRINGS.BUTTON_TARGETLOCATION_CREATESECTION;
			createSectionPopup.setText();
		}
		
		/*private function treeItemOpenHandler(evt:TreeEvent):void
		{	
			pathInput.text = XML(evt.item).@Path.toString();
		}*/
		
		/*private function treeItemCloseHandler(evt:TreeEvent):void
		{	
			var currItem:XML = XML(evt.item);
			var parentItem:XML = currItem.parent() as XML;
			if (parentItem != null)
				pathInput.text = parentItem.@Path.toString();
			else
				pathInput.text = "";
		}*/
	}
}