package framework.updater
{
	import flash.desktop.Updater;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.net.URLRequest;
	import flash.net.URLStream;
	import flash.utils.ByteArray;
	
	import framework.constants.COMMON;
	import framework.utils.Helper;
	
	import mx.controls.Alert;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.http.HTTPService;

	public class AutoUpdater extends EventDispatcher
	{
		private var versionHttpService:HTTPService = new HTTPService();
	    private var _latestVersionStr:String;
	    private var _criticalStr:String;
	    private var _featuresStr:String;
	    private var _appDownloadURLRequest:URLRequest;
	    private var _appDownloadURLStream:URLStream;
	
	    private var _config:AutoUpdateConfig;


		public function AutoUpdater(updateConfig:AutoUpdateConfig):void
	    { 
	        _config = updateConfig;
	        versionHttpService.url  = getVersionDefineFileUrl();
	       
	        versionHttpService.resultFormat = "e4x";
	        versionHttpService.addEventListener(ResultEvent.RESULT,_completeGetVersionHandler);
	        versionHttpService.addEventListener(FaultEvent.FAULT,_faultGetVersionHandler);
	
	        _appDownloadURLRequest  = new URLRequest(getApplicationFileUrl());
	       
	        _appDownloadURLStream = new URLStream();
	        _appDownloadURLStream.addEventListener(
	            Event.COMPLETE,_completeLoadAirFileHandler);
	        _appDownloadURLStream.addEventListener(
	            IOErrorEvent.IO_ERROR,_faultLoadAirFileHandler);
	        _appDownloadURLStream.addEventListener(
	            SecurityErrorEvent.SECURITY_ERROR,_faultLoadAirFileHandler);
	    }
	    
	    public function Update():void
	    {  
	    	var updater:Updater = new Updater();
	        var file:File
	            = Helper.CreateCacheDirectory(COMMON.CACHE_FOLDERNAME).resolvePath(_config.applicationFileName);
	        updater.update(file, _latestVersionStr);
	    	
	    }
	    
	     /**
	     *　
	     *
	     */
	    public function getVersion():void
	    {
	        versionHttpService.send();
	    }
	
	    /**
	     * 
	     *
	     * @param currentVer 
	     * @return 
	     *
	     */
	    public function isLatestApp(currentVer:Number):Boolean
	    {
	    //TODO:to use following line to check new version
	    //but since we are in "beta" and we have string in version number we 
	    //have commented the line
	   
	   //Note: Version us availble in the format #.#.#.#
	   //the first 2 digits determing the app version
	   //the second 2 determines the schema
	   //we extract the app version to decide to go for update or not
	    
	    if(Number(currentVer) < Number(_latestVersionStr.substring(0,_latestVersionStr.indexOf(".",_latestVersionStr.indexOf(".",0)+1))))	      
	            return false;
	        else
	            return true;
	    }
	
	    /**
	     * 
	     *
	     * @return 
	     *
	     */
	    public function getLatestVersionStr():String
	    {
	        return _latestVersionStr;
	    }
	
	    /**
	     * AIR
	     *
	     */
	    public function loadAirFile():void
	    {	        
	        _appDownloadURLStream.load(_appDownloadURLRequest);
	    }
	
	    /**
	     * 
	     *
	     * @return 
	     *
	     */
	    public function getVersionDefineFileUrl():String
	    {
	        return      _config.updateServerDomain
	                    +"/"+ _config.updateServerPath
	                    +"/"+ _config.versionDefineFileName;
	    }
	
	    
	    private function getApplicationFileUrl():String
	    {
	        return      _config.updateServerDomain
	                    +"/"+ _config.updateServerPath
	                    +"/"+ _config.applicationFileName;
	    }
	
	    //--------------------------------------------------------------------------
	    //
	    //  Event handlers
	    //
	    //--------------------------------------------------------------------------
	
	    //----------------------------------
	    // 
	    //----------------------------------
	    private function _completeGetVersionHandler(event:ResultEvent):void
	    {
	        var resultxml:XML = new XML(event.result);
	        _latestVersionStr = resultxml.AvailableVersion;
			_criticalStr = resultxml.Critical;
			_featuresStr = resultxml.Features;
	        
	        this.dispatchEvent(
	            new AutoUpdateEvent(AutoUpdateEvent.GET_VERSION_COMPLETE));
	    }
	
	    //----------------------------------
	    // 
	    //----------------------------------
	    private function _faultGetVersionHandler(event:FaultEvent):void
	    {	       
	        this.dispatchEvent(
	            new AutoUpdateEvent(AutoUpdateEvent.GET_VERSION_FAULT));
	    }
	
	    //----------------------------------
	    // AIR
	    //----------------------------------
	    private function _faultLoadAirFileHandler(event:Event):void
	    {	        
	        this.dispatchEvent(
	            new AutoUpdateEvent(AutoUpdateEvent.UPDATE_AIRFILE_FAULT));
	    }
	
	    //----------------------------------
	    // AIR
	    //----------------------------------
	    private function _completeLoadAirFileHandler(event:Event):void
	    {	        
	        var fileData:ByteArray = new ByteArray();
	        _appDownloadURLStream.readBytes(
	            fileData
	            , 0
	            , _appDownloadURLStream.bytesAvailable);
	
	        var file:File
	            = Helper.CreateCacheDirectory(COMMON.CACHE_FOLDERNAME).resolvePath(_config.applicationFileName);
	        var fileStream:FileStream = new FileStream();
	        fileStream.addEventListener(Event.CLOSE, _completeWriteFileHandler);
	        fileStream.addEventListener(IOErrorEvent.IO_ERROR, _faultWriteFileHandler);
	        fileStream.openAsync(file, FileMode.WRITE);
	        fileStream.writeBytes(fileData, 0, fileData.length);
	        fileStream.close();
	    }
	
	    //----------------------------------
	    // 
	    //----------------------------------
	    private function _completeWriteFileHandler(event:Event):void
	    {	   
	    	AssetlinkDesktop.da.DataAccessUpdate("UPDATE AUTOUPDATE SET VALUE='false' where NAME='Updated'");
	    	AssetlinkDesktop.da.DataAccessUpdate("UPDATE AUTOUPDATE SET VALUE='"+_latestVersionStr+"' where NAME='NewVersion'"); 
	    	AssetlinkDesktop.da.DataAccessUpdate("UPDATE AUTOUPDATE SET VALUE='"+_criticalStr+"' where NAME='Critical'"); 
	    	AssetlinkDesktop.da.DataAccessUpdate("UPDATE AUTOUPDATE SET VALUE='"+_featuresStr+"' where NAME='Features'");  
	    }
	
	    //----------------------------------
	    // 
	    //----------------------------------
	    private function _faultWriteFileHandler(event:IOErrorEvent):void
	    {
	        this.dispatchEvent(
	            new AutoUpdateEvent(AutoUpdateEvent.UPDATE_AIRFILE_FAULT));
	    }



		
	}
}