package UI
{
	import assets.ButtonEx;
	import assets.multiselect.TreeViewItemRenderer;
	import assets.treecheckbox.TreeCheckBox;
	
	import constants.STRINGS;	
	import events.FacetSeriesChangeEvent;
	
	import flash.events.MouseEvent;
	
	import mx.core.UIComponent;
	import mx.events.TreeEvent;
	
	public class FacetSeriesTree extends UIComponent
	{
		private var tree:TreeCheckBox;
		private var cancelBtn:ButtonEx;
		private var saveBtn:ButtonEx;
		
		private var treeviewIR:TreeViewItemRenderer;
		private var fieldData:String;
		private var mulSelSep:String = STRINGS.MULTISELECT_SEPERATOR;
		
		private var dialogWidth:int = 250;
		private var dialogHeight:int = 300;
		private var dialogLocX:int = 0;
		private var dialogLocY:int = 0;
		
		public function FacetSeriesTree()
		{
			super();
			tree = new TreeCheckBox();
		}
		
		public function initTree(treexml:XML, _treeviewIR:TreeViewItemRenderer, selval:String):void
		{	
			fieldData = selval;
			if (fieldData.length == 0)
				fieldData = mulSelSep;
			
			if (fieldData != mulSelSep)
				GetSelectedNodes(treexml.TreeNode);
			
			treeviewIR = _treeviewIR;
			tree.labelField = "@Text";
			tree.checkField = "@Checked";
			tree.dataProvider = treexml;
		}
		
		public function Show():void
		{
			this.visible = true;
		}
		
		public function Hide():void
		{
			this.visible = false;
		}
		
		protected override function createChildren():void
		{
			super.createChildren();
			
			tree.height =  250;
			tree.width = 230;
			tree.styleName = "biTree";
			tree.horizontalScrollPolicy = "ON";
			this.addChild(tree);
			
			tree.addEventListener("itemCheck", onTreeCheckBoxCheck);
			
			saveBtn = new ButtonEx();
			saveBtn.label = STRINGS.BUTTON_TREEVIEW_SAVE;
			saveBtn.setActualSize(50,25);
			this.addChild(saveBtn);
			
			saveBtn.addEventListener(MouseEvent.CLICK,saveClickHandler);
			
			cancelBtn = new ButtonEx();
			cancelBtn.label = STRINGS.BUTTON_TREEVIEW_CANCEL;
			cancelBtn.setActualSize(60,25);
			this.addChild(cancelBtn);
			
			cancelBtn.addEventListener(MouseEvent.CLICK,cancelClickHandler);	
		}
		
		protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{	
			
			this.graphics.clear();
			this.graphics.beginFill(0x313131,0.7);
			this.graphics.drawRect(0, 0, unscaledWidth, unscaledHeight);
			
			this.graphics.beginFill(0xA0A0A0,1);
			dialogLocX = (unscaledWidth-dialogWidth)/2;
			dialogLocY = (unscaledHeight-dialogHeight)/2;	
			this.graphics.drawRoundRect(dialogLocX -1,dialogLocY -1, dialogWidth+2, dialogHeight+2,25,25);
			
			this.graphics.beginFill(0x313131,1);
			this.graphics.drawRoundRect(dialogLocX ,dialogLocY , dialogWidth, dialogHeight,25,25);

			tree.x = (unscaledWidth - tree.width)/2;
			tree.y = dialogLocY + 10;//(unscaledHeight - tree.height)/2;
			
			saveBtn.x = dialogLocX +(dialogWidth-cancelBtn.width-saveBtn.width) -5 - 5;
			saveBtn.y = dialogLocY + dialogHeight - saveBtn.height -5;
			
			cancelBtn.x = dialogLocX +(dialogWidth-cancelBtn.width) -5;
			cancelBtn.y = dialogLocY + dialogHeight - cancelBtn.height -5;
		}
		
		private function cancelClickHandler(evt:MouseEvent):void
		{
			this.visible = false;			
		}
		
		private function saveClickHandler(evt:MouseEvent):void
		{	
			if (fieldData == mulSelSep)
				fieldData = "";
			treeviewIR.dispatchEvent(new FacetSeriesChangeEvent(FacetSeriesChangeEvent.TREESELECTION_CHANGE, fieldData));
			this.visible = false;
		}
		
		private function onTreeCheckBoxCheck(event: TreeEvent): void
		{
			var currXML:XML = event.item as XML;
			var tree:TreeCheckBox = event.target as TreeCheckBox;
			
			if (XML(currXML.parent()).toString() == "")
			{
				currXML.@Checked = "2";
				return;
			}
						
			var currValue:String = currXML.@Text;
			
			if (currXML.@Checked == "1")
			{	
				if (fieldData.indexOf(currValue + mulSelSep) < 0)
					fieldData += currValue + mulSelSep;
			}
			else
			{
				if (fieldData.indexOf(currValue+mulSelSep) >= 0)
					fieldData = fieldData.replace(currValue + mulSelSep,"");
			}
		}
		
		private function GetSelectedNodes(treeNodeList:XMLList):void
		{
			var selectedNodeList:XMLList;
			var selectedNode:XML;
			var treeSelData:String = fieldData;
			var tempTreeStr:String = mulSelSep;
			while (treeNodeList != null && treeNodeList.length() > 0)
			{	
				selectedNodeList = treeNodeList.(treeSelData.indexOf(@Text)>=0);
				if (selectedNodeList != null && selectedNodeList.length() > 0)
				{	
					//selectedNode = selectedNodeList[0];
					for each (selectedNode in selectedNodeList)
					{
						selectedNode.@Checked = "1";
						tempTreeStr += selectedNode.@Text + mulSelSep;
					}
					if (tempTreeStr == treeSelData)
						break;
				}
				treeNodeList = treeNodeList.TreeNode;
			}
		}
	}
}