package UI
{
	import assets.ButtonEx;
	import assets.multiselect.TreeViewItemRenderer;
	import assets.treecheckbox.TreeCheckBox;
	
	import constants.STRINGS;
	
	import events.FacetSeriesChangeEvent;
	import events.FormEvents;
	
	import flash.events.MouseEvent;
	
	import mx.core.UIComponent;
	import mx.events.TreeEvent;
	import mx.utils.StringUtil;
	
	import utils.Helper;
	
	public class MultiSelectTree extends UIComponent
	{
		private var tree:TreeCheckBox;
		private var cancelBtn:ButtonEx;
		private var saveBtn:ButtonEx;
		
		private var src:*;
		private var fieldData:String;
		private var selData:String;
		private var oldfieldData:String;
		private var mulSelSep:String = STRINGS.MULTISELECT_SEPERATOR;
		
		private var dialogWidth:int = 250;
		private var dialogHeight:int = 300;
		private var dialogLocX:int = 0;
		private var dialogLocY:int = 0;		
		
		private var fieldDisplayData:String;
		
		public function MultiSelectTree()
		{
			super();
			tree = new TreeCheckBox();
		}
		
		public function initTree(treexml:XML, _src:*, selval:String, actualVal:String, treeDisplayVal:String=""):void
		{	
			//fieldData = selval;
			selData = selval;
			fieldData = actualVal;			
			fieldDisplayData = treeDisplayVal;
			src = _src;
			
			//if(treexml == null)
			//	return;
			
			if (fieldData.length == 0)
				fieldData = mulSelSep;
			
			if (fieldDisplayData.length == 0)
				fieldDisplayData = ",";
			
			oldfieldData = fieldData;	
			if(treexml != null)
			{
				unCheckAllTreeNodes(treexml.TreeNode);
				if (fieldData != mulSelSep)
					GetSelectedNodes(treexml.TreeNode);			
			}
			
			//tree.labelField = "@Text";
			tree.labelFunction = Helper.getNamedHierarchyDisplayValue;
			tree.checkField = "@Checked";
			tree.dataProvider = treexml;			
		}
		
		public function Show():void
		{
			this.visible = true;
		}
		
		public function Hide():void
		{
			this.visible = false;
		}
		
		public function setText():void
		{
			saveBtn.label = STRINGS.BUTTON_TREEVIEW_SAVE;
			cancelBtn.label = STRINGS.BUTTON_TREEVIEW_CANCEL;
		}
		
		protected override function createChildren():void
		{
			super.createChildren();
			
			tree.height =  250;
			tree.width = 230;
			tree.styleName = "biTree";
			tree.horizontalScrollPolicy = "ON";
			this.addChild(tree);
			
			tree.addEventListener("itemCheck", onTreeCheckBoxCheck);
			
			saveBtn = new ButtonEx();
			saveBtn.label = STRINGS.BUTTON_TREEVIEW_SAVE;
			saveBtn.setActualSize(80,25);
			this.addChild(saveBtn);
			
			saveBtn.addEventListener(MouseEvent.CLICK,saveClickHandler);
			
			cancelBtn = new ButtonEx();
			cancelBtn.label = STRINGS.BUTTON_TREEVIEW_CANCEL;
			cancelBtn.setActualSize(80,25);
			this.addChild(cancelBtn);
			
			cancelBtn.addEventListener(MouseEvent.CLICK,cancelClickHandler);	
		}
		
		protected override function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{	
			
			this.graphics.clear();
			this.graphics.beginFill(0x313131,0.7);
			this.graphics.drawRect(0, 0, unscaledWidth, unscaledHeight);
			
			this.graphics.beginFill(0xA0A0A0,1);
			dialogLocX = (unscaledWidth-dialogWidth)/2;
			dialogLocY = (unscaledHeight-dialogHeight)/2;	
			this.graphics.drawRoundRect(dialogLocX -1,dialogLocY -1, dialogWidth+2, dialogHeight+2,25,25);
			
			this.graphics.beginFill(0x313131,1);
			this.graphics.drawRoundRect(dialogLocX ,dialogLocY , dialogWidth, dialogHeight,25,25);

			tree.x = (unscaledWidth - tree.width)/2;
			tree.y = dialogLocY + 10;//(unscaledHeight - tree.height)/2;
			
			saveBtn.x = dialogLocX +(dialogWidth-cancelBtn.width-saveBtn.width) -5 - 5;
			saveBtn.y = dialogLocY + dialogHeight - saveBtn.height -5;
			
			cancelBtn.x = dialogLocX +(dialogWidth-cancelBtn.width) -5;
			cancelBtn.y = dialogLocY + dialogHeight - cancelBtn.height -5;
		}
		
		private function cancelClickHandler(evt:MouseEvent):void
		{
			this.visible = false;			
		}
		
		private function saveClickHandler(evt:MouseEvent):void
		{	
			
			if (oldfieldData != fieldData)
			{	
				if (fieldData == mulSelSep)
					fieldData = "";
				
				if(fieldDisplayData != "")				
				{
					if(fieldDisplayData.lastIndexOf(",", fieldDisplayData.length) == fieldDisplayData.length - 1)
						fieldDisplayData = fieldDisplayData.substr(0,fieldDisplayData.length - 1); //Remove last comma
					if(fieldDisplayData.indexOf(",", 0) == 0)
						fieldDisplayData = fieldDisplayData.substr(1,fieldDisplayData.length);//Remove first comma
				}	
				
				var eventData:String = fieldData + "$DATA$" +fieldDisplayData;
					
				if (src is TreeViewItemRenderer)
					src.dispatchEvent(new FacetSeriesChangeEvent(FacetSeriesChangeEvent.TREESELECTION_CHANGE, eventData));
				else
					src.dispatchEvent(new FormEvents(FormEvents.TREESELECTION_CHANGE, eventData));
			}
			this.visible = false;
		}
		
		private function onTreeCheckBoxCheck(event: TreeEvent): void
		{
			var displayvalueForSelectedItem:String = null;
			var currXML:XML = event.item as XML;
			var tree:TreeCheckBox = event.target as TreeCheckBox;
			
			if (XML(currXML.parent()).toString() == "")
			{
				currXML.@Checked = "2";
				return;
			}
			
			var currValue:String = "";	
			if (src.name =="Path")
			{		
				var parentArray:Array = new Array();
				var parentDispValueArray:Array = new Array();
								
				parentArray.push(currXML.@Text);
				
				if(currXML.@DisplayTextID != null && currXML.@DisplayTextID != "")
				{
					displayvalueForSelectedItem = Helper.GetLocalizedString(currXML.@DisplayTextID);
					if(displayvalueForSelectedItem == currXML.@DisplayTextID)
						displayvalueForSelectedItem = currXML.@Text;
				}	
				else
					displayvalueForSelectedItem = currXML.@Text;
				
				parentDispValueArray.push(displayvalueForSelectedItem);
				
				var obj:Object =tree.getParentItem(event.item);				  
				  
				  while(obj!=null)
				  {
				  	parentArray.push(obj.@Text);
					
					//Populate dispay value array for parent nodes
					if(obj.@DisplayTextID != null && obj.@DisplayTextID != "")
					{
						displayvalueForSelectedItem = Helper.GetLocalizedString(obj.@DisplayTextID);
						if(displayvalueForSelectedItem == obj.@DisplayTextID)
							displayvalueForSelectedItem = obj.@Text;
					}
					else
						displayvalueForSelectedItem = obj.@Text;
					
					parentDispValueArray.push(displayvalueForSelectedItem);
					
				  	obj=tree.getParentItem(obj);
				  	
				  }
				
				  var path:String="";
				  for(var i:int=parentArray.length-1;i>0;i--){
				  	if(path != "")
				  	{
				  		path = path + "\\";
				  	}
				  	path = path + parentArray[i-1] ;
				  }
				  currValue = path;
				  
				  path="";
				  for(var j:int=parentDispValueArray.length-1;j>0;j--)
				  {
					  if(path != "")
						  path = path + "\\";
					  
					  path = path + parentDispValueArray[j-1] ;
				  }
				  displayvalueForSelectedItem = path;
			}
			else
			{
				currValue = currXML.@Text;
				if(currXML.@DisplayTextID != null && currXML.@DisplayTextID != "")
				{
					displayvalueForSelectedItem = Helper.GetLocalizedString(currXML.@DisplayTextID);
					if(displayvalueForSelectedItem == currXML.@DisplayTextID)
						displayvalueForSelectedItem = currValue;
				}	
				else
					displayvalueForSelectedItem = currValue;
			}
			 
			
			if (currXML.@Checked == "1")
			{	
				if (fieldData.indexOf(currValue + mulSelSep) < 0)
				{
					fieldData += StringUtil.trim(currValue) + mulSelSep;
					fieldDisplayData += StringUtil.trim(displayvalueForSelectedItem) + ",";
				}			
			}
			else
			{
				currXML.@Checked == "0";
				if (fieldData.indexOf(mulSelSep+currValue+mulSelSep) >= 0)
				{
					fieldData = fieldData.replace(mulSelSep+currValue + mulSelSep,mulSelSep);
					fieldDisplayData = fieldDisplayData.replace( "," + displayvalueForSelectedItem + "," , ",");
				}	
			}
			
		}
		
		private function unCheckAllTreeNodes(treeNodeList:XMLList):void
		{
			var treeNode:XML = null;
			
			if(treeNodeList != null && treeNodeList.length() > 0)
			{
				for(var nodeCnt:Number=0; nodeCnt < treeNodeList.length(); nodeCnt++)
				{
					treeNode = XML(treeNodeList[nodeCnt]);
					treeNode.@Checked = "0";	
					
					var treeChildNodeList:XMLList = treeNode.TreeNode;
					if(treeChildNodeList != null && treeChildNodeList.length() > 0)
						unCheckAllTreeNodes(treeChildNodeList);
				}			
			}
		}
		private function GetSelectedNodes(treeNodeList:XMLList):void
		{
			var selectedNodeList:XMLList;
			var selectedNode:XML;
			var treeSelData:String = selData;
			var tempTreeStr:String = mulSelSep;
			var selectedValues:Array;
			var treeNode:XML= null;			
			var currentValue:String = null;
			var tempTreeNodeList:XMLList = null;
			var returnValue:String = null;
			
			selectedValues = fieldData.split(mulSelSep);
			if(treeNodeList != null && treeNodeList.length() > 0)
			{
				for(var selValCnt:Number =0 ; selValCnt < selectedValues.length; selValCnt++)
				{
					tempTreeNodeList = treeNodeList;
					currentValue = selectedValues[selValCnt];
					
					if(currentValue != "")
					{
						//NH by Path
						if (src.name =="Path")					
							setSelectedPathTreeNode(currentValue, tempTreeNodeList);			
						else //NH by Value
							setSelectedTreeNode(currentValue, tempTreeNodeList);
					}
				}
			}			
		}
		
		private function setSelectedTreeNode(currentValue:String, TreeNodeList:XMLList):void
		{
			var treeNode:XML = null;
			var displayTextID:String = null;
			
			for(var nodeCnt:Number=0; nodeCnt < TreeNodeList.length(); nodeCnt++)
			{
				treeNode = XML(TreeNodeList[nodeCnt]);
				if(treeNode.@Text == currentValue)
				{
					displayTextID = treeNode.@DisplayTextID;	
					treeNode.@Checked = "1";
					if(displayTextID != null && displayTextID != "")
					{
						if(displayTextID != Helper.GetLocalizedString(displayTextID))
							fieldDisplayData += Helper.GetLocalizedString(displayTextID) + ",";
						else
							fieldDisplayData += currentValue + ",";
					}
					else
						fieldDisplayData += currentValue + ",";
				}
				
				setSelectedTreeNode(currentValue, treeNode.TreeNode);
			}
		}
		
		private function setSelectedPathTreeNode(currentValue:String, TreeNodeList:XMLList):void
		{
			var pathSeperator:String = "\\";
			var pathValues:Array = currentValue.split(pathSeperator);
			var currentNHNodeValue:String = null;
			var tempTreeNodeList:Array = new Array();
			var isLeafNode:Boolean = false;
			
			for(var i:Number=0;i<TreeNodeList.length();i++)
				tempTreeNodeList[i] = XML(TreeNodeList[i]);
			
			for(var cnt:Number = 0; cnt < pathValues.length; cnt++)
			{
				if(cnt == pathValues.length - 1)
					isLeafNode = true;
				
				currentNHNodeValue = pathValues[cnt];
				tempTreeNodeList = getCurrentNHValueNodeList(tempTreeNodeList, currentNHNodeValue,isLeafNode);
			}
		}
		
		private function getCurrentNHValueNodeList(TreeNodeArray:Array, currentNHNodeValue:String, isLeafNode:Boolean):Array
		{
			var TreeNode:XML = null;
			var updatedTreeNodeArray:Array = new Array();
			var cnt:Number = 0 ;
			var displayTextID:String = null;
			
			for(var i:Number=0;i<TreeNodeArray.length;i++)
			{
				TreeNode = XML(TreeNodeArray[i]);
				if(TreeNode.@Text == currentNHNodeValue)
				{
					displayTextID = TreeNode.@DisplayTextID;
					if(isLeafNode)
					{
						TreeNode.@Checked = "1";	
						if(displayTextID != null && displayTextID != "")
						{
							if(displayTextID != Helper.GetLocalizedString(displayTextID))
								fieldDisplayData += Helper.GetLocalizedString(displayTextID) + ",";
							else
								fieldDisplayData += currentNHNodeValue + ",";
						}
						else
							fieldDisplayData += currentNHNodeValue + ",";
					}
					else
					{
						var tmpList:XMLList = TreeNode.TreeNode;
						if(tmpList.length() > 0 )
						{
							for(var j:Number=0;j<tmpList.length();j++)
							{	
								updatedTreeNodeArray[cnt] =  XML(tmpList[j]);
								cnt++;
							}
						}
						if(displayTextID != null && displayTextID != "")
						{
							if(displayTextID != Helper.GetLocalizedString(displayTextID))
								fieldDisplayData += Helper.GetLocalizedString(TreeNode.@DisplayTextID) + "\\";
							else
								fieldDisplayData += currentNHNodeValue + "\\";
						}
						else
							fieldDisplayData += currentNHNodeValue + "\\";
					}
				}
			}
			return updatedTreeNodeArray;
		}
	}
}