package utils
{
	import Assetlink.Solutions.Formatting.NumberFormatting;
	
	import constants.STRINGS;
	
	import infrastructure.statemanagement.StateMgr;
	
	import mx.containers.HBox;
	import mx.controls.Alert;
	import mx.controls.ComboBox;
	import mx.controls.DateField;
	import mx.controls.Text;
	import mx.controls.TextArea;
	import mx.controls.TextInput;
	import mx.events.ValidationResultEvent;
	import mx.validators.EmailValidator;
	import mx.validators.NumberValidator;
	import mx.validators.StringValidator;
	import mx.validators.Validator;
	
	public class Validators
	{	
		private var isDataValid:Boolean = true;
				
		public function ValidateField(src:*, currField:XML):Boolean
		{			
			if (String(currField.@DataType).toUpperCase() == "INTEGER" || String(currField.@DataType).toUpperCase() == "FLOAT" || String(currField.@DataType).toUpperCase() == "CURRENCY")
				isDataValid = ValidateNumericField(src, currField);
			else if (String(currField.@DataType).toUpperCase() == "STRING" || String(currField.@DataType).toUpperCase() == "TEXT" || String(currField.@DataType).toUpperCase() == "DATETIME")
				isDataValid = ValidateStringField(src, currField);				
			return isDataValid;
		}
		
		public function ValidateNumericField(src:*, currField:XML):Boolean
		{	
			var _formValidators:Array = [];
			
			var numValidator:NumberValidator = new NumberValidator();
			numValidator.requiredFieldError = STRINGS.MANADATORY_FIELD_MESSAGE;
			numValidator.precisionError = STRINGS.FIELD_PRECISION_MESSAGE; 
			numValidator.exceedsMaxError = STRINGS.FIELD_MAXEXCEED_MESSAGE;
			numValidator.integerError = STRINGS.FIELD_INTEGERERROR_MESSAGE;   
			numValidator.invalidCharError = STRINGS.FIELD_INVALIDCHARERROR_MESSAGE; 
			numValidator.invalidFormatCharsError = STRINGS.FIELD_INVALIDFORMATCHARSERROR_MESSAGE; 
			numValidator.lowerThanMinError = STRINGS.FIELD_LOWERTHANMINERROR_MESSAGE;
			numValidator.negativeError = STRINGS.FIELD_NEGATIVEERROR_MESSAGE;
			numValidator.separationError = STRINGS.FIELD_SEPARATIONERROR_MESSAGE;			
			
			//precision, decimalSeparatorTo, thousandsSeparatorTo
			var currentLanguage:String = StateMgr.GetInstance().Language;
			var currentCulture:String = StateMgr.GetInstance().CurrentCulture;
			var FormatXml:XML = XML(StateMgr.GetInstance().FormatXml);
			var CurrentNumberFormatting:XML = null;
			
			if(currField.@DataType.toUpperCase() == "FLOAT")
				CurrentNumberFormatting = XML(FormatXml.language.(@name == currentLanguage).FormattingTypes.FloatFormatting);
			else if(currField.@DataType.toUpperCase() == "CURRENCY")
				CurrentNumberFormatting = XML(FormatXml.language.(@name == currentLanguage).FormattingTypes.CurrencyFormatting);
			else
				CurrentNumberFormatting = XML(FormatXml.language.(@name == currentLanguage).FormattingTypes.NumberFormatting);
			
			var ObjNumberFormat:NumberFormatting  = new NumberFormatting(CurrentNumberFormatting);
			numValidator.precision = ObjNumberFormat.DecimalDigits;
			numValidator.decimalSeparator = ObjNumberFormat.DecimalSeparator;
			numValidator.thousandsSeparator = ObjNumberFormat.GroupSeparator;
			
			numValidator.source = src;
			if (src is TextInput || src is TextArea || src is Text)
				numValidator.property = "text";
			else if (src is ComboBox)
				numValidator.property = "selectedLabel";
			
			ValidateMandatoryField(src, currField, numValidator);			
			ValidateFloatField(currField, numValidator);			
			
			if (currField.Validators != null && String(currField.Validators).length > 0)
			{	
				var validatorNodeList:XMLList = currField.Validators.Validator;
				var validatorNode:XML;
				for  each (validatorNode in validatorNodeList)
				{
					if (String(validatorNode.@Type).toUpperCase() == "LENGTH")
					{
						var stringValidator:StringValidator = new StringValidator();
						stringValidator.tooLongError = STRINGS.FIELD_TOOLONGERROR_MESSAGE;
						stringValidator.tooShortError = STRINGS.FIELD_TOOSHORTERROR_MESSAGE;
						stringValidator.requiredFieldError = STRINGS.MANADATORY_FIELD_MESSAGE;
						stringValidator.source = src;
						stringValidator.property = "text";
						ValidateLength(validatorNode, stringValidator);
						_formValidators.push(stringValidator);
					}
					if (String(validatorNode.@Type).toUpperCase() == "RANGE")
						ValidateRange(validatorNode, numValidator);
				}
			}
			
			_formValidators.push(numValidator);
			isDataValid = true;
			var failedValidations:Array = new Array();
			failedValidations = Validator.validateAll(_formValidators);
			for each (var validationResultEvent:ValidationResultEvent in failedValidations)
			{
				isDataValid = false;
				RemoveRepeatingErrorMessages(validationResultEvent);
			}

			return isDataValid; 
		}
		
		//Reference Link:http://stackoverflow.com/questions/3576303/flex-3-5-validator-error-message-repeats (hackForSDK35Bug22911ToRemoveRepeatingErrorMessages)
		private function RemoveRepeatingErrorMessages(failedValidation:ValidationResultEvent):void
		{
			var uiComponent:Object = failedValidation.target.source;
			var errorString:String = uiComponent.errorString as String;
			uiComponent.errorString = errorString.split("\n")[0];
		}
		
		public function ValidateStringField(src:*, currField:XML):Boolean
		{
			var _formValidators:Array = [];
			
			var stringValidator:StringValidator = new StringValidator();
			stringValidator.source = src;
			stringValidator.tooLongError = STRINGS.FIELD_TOOLONGERROR_MESSAGE;
			stringValidator.tooShortError = STRINGS.FIELD_TOOSHORTERROR_MESSAGE;
			stringValidator.requiredFieldError = STRINGS.MANADATORY_FIELD_MESSAGE;
			if (src is TextInput || src is TextArea || src is Text)			
				stringValidator.property = "text";
			else if (src is ComboBox)
				stringValidator.property = "selectedLabel";
			else if (src is DateField)
				stringValidator.property = "selectedDate";
				
			ValidateMandatoryField(src, currField, stringValidator);
			
			if (currField.Validators != null && String(currField.Validators).length > 0)
			{
				var validatorNodeList:XMLList = currField.Validators.Validator;
				var validatorNode:XML;
				for  each (validatorNode in validatorNodeList)
				{
					if (String(validatorNode.@Type).toUpperCase() == "LENGTH")
						ValidateLength(validatorNode, stringValidator);
					if (String(validatorNode.@Type).toUpperCase() == "EMAIL")
						ValidateEmail(src, currField);
				}
			}			
			
			isDataValid = true;
			_formValidators.push(stringValidator);
			var failedValidations:Array = new Array();
			failedValidations = Validator.validateAll(_formValidators);
			for each (var validationResultEvent:ValidationResultEvent in failedValidations)
			{
				isDataValid = false;
				RemoveRepeatingErrorMessages(validationResultEvent);
			}
			return isDataValid;
		}
		
		public function ValidateMandatoryField(src:*, currField:XML, validator:*):void
		{
			//validator.source = src; 
			if (String(currField.@IsMandatory).toUpperCase() == "YES" || String(currField.@IsMandatory).toUpperCase() == "TRUE")
			{
				validator.required = true;
			}
			else
			{
				validator.required = false;
			}
		}
		
		public function ValidateFloatField(currField:XML, numValidator:NumberValidator):void
		{
			if (currField.@DigitsAfterDecimal != null && String(currField.@DigitsAfterDecimal).length > 0)
				numValidator.precision = currField.@DigitsAfterDecimal;
		}
		
		public function ValidateLength(validatorNode:XML, stringValidator:StringValidator):void
		{
			if (validatorNode.@MaxLength != null && String(validatorNode.@MaxLength).length > 0)
				stringValidator.maxLength = validatorNode.@MaxLength;			
		}
		
		public function ValidateRange(validatorNode:XML, numValidator:NumberValidator):void
		{	
			numValidator.minValue = validatorNode.@MinValue;
		 	numValidator.maxValue = validatorNode.@MaxValue;			
		}
		
		public function ValidateEmail(src:*, currField:XML):void
		{
			var emailValidator:EmailValidator = new EmailValidator();
			emailValidator.source = src;
			emailValidator.property = "text";
		}
		
		public function ValidateHBox(hbox:HBox, fieldData:String):Boolean
        {   			
        	if (fieldData.length == 0)
	  		{			  												  									  
	  			hbox.errorString = STRINGS.MANADATORY_FIELD_MESSAGE;
	  			hbox.setStyle("borderStyle","1px solid #FF0000");
	  			isDataValid = false;
	  		}
	  		else
	  		{
	  			hbox.errorString = "";
	  			hbox.setStyle("borderStyle","none");
	  			isDataValid = true;
	  		}
	  		return isDataValid;
        }
        
        private function ValidationResult(evt:ValidationResultEvent):Boolean
		{	
			isDataValid = true;
			if (evt.type == ValidationResultEvent.INVALID)
			{
				isDataValid = false;
			}			
			
			return isDataValid;
		}
		
		
	}
}